<html>
    <head>
        <title></title>
    </head>
    <body style="margin: 0px;padding: 5px;width: 60mm;height: 58mm;padding-right: 5px;">
            <h2 style="margin: 0;">Попона денниковая  Cosy Stable Lite 100g </h1>
            140<br>
            <b>RAMBOMICKLEM861557</b>
            
        <img src="/Users/p141592/Projects/gdshop/glib2/gdshoplib/templates/qr.png" width="80px" style="position: absolute; bottom: 5px; left: 5px;">
        <img src="/Users/p141592/Projects/gdshop/glib2/gdshoplib/templates/name.png" width="130px" style="position: absolute; bottom: 30px; right: 12px;margin: 0;font-size: 15px;">
    </body>
</html>