class Delivery:
    # Расчет веса
    # Расчет упаковки
    # Расчет цены
    # Печать накладной
    # Выбор провайдера
    # Установка ID
    # Возврат информации о доставке
    # Hook для статуса
    ...
